<?php
/**
 * @file
 * Connector to OHT Service.
 */

/**
 * OHT connector class.
 */
class OHTConnector {

  private $sandbox;
  private $publicKey;
  private $secretKey;

  private $responseCode;
  private $responseStatus;

  const SANDBOX_URL = 'https://sandbox.onehourtranslation.com/api/2/';
  const PRODUCTION_URL = 'https://api.onehourtranslation.com/api/2/';

  public $debug;

  /**
   * @param string $public_key
   *   OHT public key.
   * @param $secret_key
   *   OHT secret key.
   * @param bool $sandbox
   *   Flag to determine if to connect to sandbox.
   */
  function __construct($public_key, $secret_key, $sandbox = FALSE) {
    $this->publicKey = $public_key;
    $this->secretKey = $secret_key;
    $this->sandbox = $sandbox;

    $this->debug = variable_get('tmgmt_oht_debug', FALSE);
  }

  /**
   * Creates a text resource at OHT.
   *
   * @param string $text
   *   Text to be translated.
   *
   * @return array
   *   OHT uuid of the resource.
   */
  public function uploadTextResource($text) {
    $params['text'] = $text;
    return $this->getRequest('resources/text', 'POST', $params);
  }

  /**
   * Downloads resource.
   *
   * @param string $oht_uuid
   *
   * @return array
   *   Resource xml.
   */
  public function getResourceDownload($oht_uuid, $project_id = NULL) {
    return $this->getRequest('resources/' . $oht_uuid . '/download', 'GET', ($project_id) ? array('project_id' => $project_id) : array(), TRUE);
  }

  /**
   * Returns supported language pairs.
   *
   * @return mixed
   *   The list of supported language pairs.
   */
  public function getSupportedLanguagePairs() {
    return $this->getRequest('discover/language_pairs', 'GET', array(), TRUE);
  }

  /**
   * Returns supported languages.
   *
   * @return mixed
   *   The list of supported languages.
   */
  public function getSupportedLanguages() {
    return $this->getRequest('discover/languages', 'GET', array(), TRUE);
  }

  /**
   * Creates new translation project at OHT.
   *
   * @param int $tjiid
   *   Translation job item id.
   * @param string $source_language
   *   Source language.
   * @param string $target_language
   *   Target language.
   * @param string $oht_uuid
   *   OHT uuid.
   * @param string $notes
   *   Notes to be sent with the job.
   * @param string $expertise
   *   Expertise code.
   * @param array $params
   *   Additional params.
   *
   * @return array
   *   OHT project data.
   */
  public function newTranslationProject($tjiid, $source_language, $target_language, $oht_uuid, $notes = NULL, $expertise = NULL, $params = array()) {
    $params['source_language'] = $source_language;
    $params['target_language'] = $target_language;
    $params['sources'] = $oht_uuid;
    $params['notes'] = $notes;
    $params['callback_url'] = url('tmgmt_oht_callback', array('absolute' => TRUE));
    $params['custom0'] = $tjiid;
    $params['custom1'] = tmgmt_oht_hash($tjiid);

    if (!empty($expertise)) {
      $params['expertise'] = $expertise;
    }

    return $this->getRequest('projects/translation', 'POST', $params);
  }

  /**
   * Gets OHT project data.
   *
   * @param int $project_id
   *   OHT project id.
   *
   * @return array
   *   Project info.
   */
  public function getProjectDetails($project_id) {
    return $this->getRequest('projects/' . $project_id, 'GET');
  }

  /**
   * Create new comment to project.
   *
   * @param int $project_id
   * @param string $content (optional)
   *
   * @return array
   *   Response.
   */
  public function addProjectComment($project_id, $content = '') {
    $params['content'] = $content;
    return $this->getRequest('projects/' . $project_id . '/comments', 'POST', $params);
  }

  /**
   * Fetch comments by a project id
   *
   * @param int $project_id
   *
   * @return array
   *   Project comments.
   */
  public function getProjectComments($project_id) {
    try {
      return $this->getRequest('projects/' . $project_id . '/comments', 'GET');
    } catch (\Exception $e) {
      return array();
    }
  }

  /**
   * Gets wordcount.
   *
   * @param string $oht_uuid
   *   OHT resource uuid.
   *
   * @return array
   *   Wordcount info.
   */
  public function getWordcount($oht_uuid) {
    $params['resources'] = $oht_uuid;
    return $this->getRequest('tools/wordcount', 'GET', $params);
  }

  /**
   * Returns account details.
   *
   * @return mixed
   *   The array containing account details.
   */
  public function getAccountDetails() {
    return $this->getRequest('account', 'GET', array());
  }

  /**
   * Gets quotation.
   *
   * @param array $resource_uuids
   *   List of OHT resource uuids.
   * @param int $word_count
   *   The number of words to translate.
   * @param string $source_language
   *   The source language.
   * @param string $target_language
   *   The target language.
   * @param string $service
   *   (optional) Service value. It can be one option from:
   *    - translation (default value)
   *    - proofreading
   *    - transproof
   *    - transcription
   * @param string $expertise
   *   (optional) Expertise code.
   * @param string $proofreading
   *   (optional) If to provide proofreading.
   * @param string $currency
   *   (optional) Currency code.
   *
   * @return mixed
   *   Quotation data.
   */
  public function getQuotation(array $resource_uuids, $word_count = 0, $source_language, $target_language, $service = 'translation', $expertise = NULL, $proofreading = NULL, $currency = NULL) {
    $params['resources'] = implode(',', $resource_uuids);
    $params['word_count'] = $word_count;
    $params['source_language'] = $source_language;
    $params['target_language'] = $target_language;
    $params['service'] = $service;
    $params['expertise'] = $expertise;
    $params['proofreading'] = $proofreading;
    $params['currency'] = $currency;

    return $this->getRequest('tools/quote', 'GET', $params);
  }

  /**
   * Returns list of expertise options.
   *
   * @param string $source_language
   *   (optional) The source language. Mandatory if target language is
   *   specified.
   * @param string $target_language
   *   (optional) The target language. Mandatory if source language is
   *   specified.
   *
   * @return array
   *   List of expertise options, keyed by their code.
   */
  public function getExpertise($source_language = NULL, $target_language = NULL) {
    $params['source_language'] = $source_language;
    $params['target_language'] = $target_language;

    return $this->getRequest('discover/expertise', 'GET', $params);
  }

  /**
   * Creates a file resource at OHT.
   *
   * @param string $xliff
   *   .XLIFF string to be translated. It is send as a file.
   * @param string $name
   *   File name of the .XLIFF file.
   *
   * @return array
   *   OHT uuid of the resource.
   */
  public function uploadFileResource($xliff, $name) {
    $boundary = '------------------------acee0aaa2902bfa2';
    $content_type = "multipart/form-data; boundary=$boundary";

    $output = "";
    $output .= "--$boundary\r\n";
    $output .= "Content-Disposition: form-data; name=\"upload\"; filename=\"$name.xliff\"\r\nContent-Type: text/plain\r\n\r\n";
    $output .= $xliff;
    $output .= "\r\n\r\n";
    $output .= "--$boundary\r\n";
    $output .= "Content-Disposition: form-data; name=\"public_key\"\r\n\r\n$this->publicKey\r\n";
    $output .= "--$boundary\r\n";
    $output .= "Content-Disposition: form-data; name=\"secret_key\"\r\n\r\n$this->secretKey\r\n";
    $output .= "--$boundary--";

    return $this->getRequest('resources/file', 'POST', $output, FALSE, $content_type);
  }

  /**
   * Processes request to OHT service.
   *
   * @param string $path
   *   Path to request.
   * @param string $method
   *   Method to use.
   * @param mixed $data
   *   Additional data sent with a request.
   * @param bool $download
   *   If we expect resource to be downloaded.
   * @param string $content_type
   *   (optional) Content-type to use.
   *
   * @return mixed
   *
   * @throws TMGMTOHTException
   */
  protected function getRequest($path, $method = 'GET', $data = array(), $download = FALSE, $content_type = 'application/x-www-form-urlencoded') {
    $options = array();

    if ($this->sandbox) {
      $url = self::SANDBOX_URL . $path;
    }
    else {
      $url = self::PRODUCTION_URL . $path;
    }

    if (is_array($data)) {
      $data['public_key'] = $this->publicKey;
      $data['secret_key'] = $this->secretKey;
    }

    // If we will send GET request - add params to the URL.
    if ($method == 'GET') {
      $url = url($url, array('query' => $data));
    }
    // Otherwise - configure $options array.
    elseif ($method == 'POST') {
      $options += array(
        'headers' => array('Content-Type' => $content_type),
        'method' => 'POST',
      );
      if (is_array($data)) {
        $options['data'] = drupal_http_build_query($data);
      }
      // Support for multipart form data.
      else {
        $options['data'] = $data;
      }
    }

    $response = drupal_http_request($url, $options);

    if ($this->debug) {
      watchdog('tmgmt oht debug', "Received response at @url <pre>@params\n\n@response</pre>",
        array('@url' => $url, '@params' => var_export($data, TRUE), '@response' => var_export($response, TRUE)));
    }

    $this->responseCode = $response->code;

    if ($response->code != 200) {
      throw new TMGMTOHTException('Unable to connect to the OHT service due to following error: @error at @url',
        array('@error' => $response->error, '@url' => $url));
    }

    // If we are expecting a download, just return received data.
    if ($download) {
      return $response->data;
    }

    $response = drupal_json_decode($response->data);

    $this->responseStatus = $response['status'];

    if ($response['status']['code'] != 0) {
      throw new TMGMTOHTException('OHT service returned validation error: #%code %error',
        array('%code' => $response['status']['code'], '%error' => $response['status']['msg']));
    }

    if (!empty($response['errors'])) {
      watchdog('tmgmt_oht', 'OHT error: @error', array('@error' => implode('; ', $response['errors'])));
      throw new TMGMTOHTException('OHT service returned following error: %error',
        array('%error' => $response['status']['msg']));
    }

    return $response['results'];
  }

  /**
   * Cancels the OHT project.
   *
   * @param int $project_id
   *   OHT project id.
   *
   * @return array
   *   The response after cancelled project.
   */
  public function cancelProject($project_id) {
    return $this->getRequest('projects/' . $project_id . '?secret_key=' . $this->secretKey . '&public_key=' . $this->publicKey, 'DELETE');
  }

}
